import React, { useContext } from "react";
import styling from "@devcake/styling";
import PropTypes from "prop-types";

type Resolver = (key: string) => string;

const context = React.createContext<Resolver>(key => styling.get(key));

export type ScopeProps = {
  children: React.ReactNode;
  overrides: { [key: string]: string };
  scope?: string;
}

const makeKey = (...parts: string[]) => parts.join(".").replace(/\.$/, "");

export const Scope: React.FunctionComponent<ScopeProps> = ({ children, overrides, scope }) => {
  const parent = useContext(context);

  const mapped = React.useMemo(() => {
    if (!scope) return overrides;
    let mapped = {};
    Object.entries(overrides).forEach(([key, value]) => mapped[makeKey(scope, key)] = value);
    return mapped;
  }, [overrides, scope]);

  const resolver: Resolver = React.useMemo(() => key => {
    if (key in mapped) return mapped[key];
    else return parent(key);
  }, [mapped, parent]);
  return <context.Provider value={resolver}>{children}</context.Provider>;
};
Scope.propTypes = {
  children: PropTypes.node.isRequired,
  overrides: PropTypes.objectOf(PropTypes.string).isRequired,
  scope: PropTypes.string
};

export const useStyle = (keys: string[], scope?: string) => {
  const resolver = useContext(context);
  if (scope) keys = keys.map(key => makeKey(scope, key));
  return keys.map(resolver);
};
